@@ -73,16 +73,20 @@ class file:
             "pp",
             True
         )
+        ea = kwargs.get(
+            "ensure_ascii",
+            False
+        )
         with open(p(fi), "w", encoding="iso-8859-1") as f:
             if objtype(pp) != "bool":
                 f.close()
                 raise InvalidPPBool("Argument \"pp\" must be bool")
             
             if pp:
-                json.dump(data, f, indent=4)
+                json.dump(data, f, indent=4, ensure_ascii=ea)
                 
             elif not pp:
-                json.dump(data, f)
+                json.dump(data, f, ensure_ascii=ea)
                 
             else:
                 raise UnknownError("¯\_(ツ)_/¯")
@@ -109,7 +113,7 @@ class file:
             os.mkdir(path)
             return 0
             
-        except FileExistsError:
+        except (FileExistsError, PermissionError):
             return 1
         
     def rmdir(self, path):
@@ -117,7 +121,15 @@ class file:
             shutil.rmtree(path)
             return 0
         
-        except FileNotFoundError:
+        except (FileNotFoundError, PermissionError):
+            return 1
+        
+    def copydir(self, path, path2):
+        try:
+            shutil.copytree(path, path2)
+            return 0
+        
+        except (FileNotFoundError, PermissionError):
             return 1
 
 file = file()
\ No newline at end of file
