@@ -1,9 +1,3 @@
-try:
-    from psutil import virtual_memory, swap_memory
-    
-except:
-    pass
-
 from .exception import *
 from .objtype import *
 
@@ -11,20 +5,26 @@ class mem:
     """Memory shtuff"""
     def mem(self):
         """Returns total, used and available memory."""
-        try:
-            yee = virtual_memory()
-            return [yee.total, yee.used, yee.free]
         
-        except NameError:
+        try:
+            from psutil import virtual_memory, swap_memory
+            
+        except (ModuleNotFoundError, ImportError):
             raise ModuleError("psutil must be installed to use beetroot.mem.mem(). Use pip install psutil or pip install beetroot[ram].")
         
+        yee = virtual_memory()
+        return [yee.total, yee.used, yee.free]
+        
     def swapmem(self):
         """Returns total, used and available swap memory."""
+        
         try:
-            yee = swap_memory()
-            return [yee.total, yee.used, yee.free]
-        
-        except NameError:
-            raise ModuleError("psutil must be installed to use beetroot.mem.swapmem(). Use pip install psutil or pip install beetroot[ram].")
+            from psutil import virtual_memory, swap_memory
+            
+        except (ModuleNotFoundError, ImportError):
+            raise ModuleError("psutil must be installed to use beetroot.mem.mem(). Use pip install psutil or pip install beetroot[ram].")
         
+        yee = swap_memory()
+        return [yee.total, yee.used, yee.free]
+                
 mem = mem()
\ No newline at end of file
